import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

public class Level
{
  private int x;
  private int y;
  private int endX;
  private int endY;
  private String next;
  private String[] enemy;
  private int tmpEnemyLineNo = 0;
  
  public Level(String level)
  {
    try
    {
      parseFile("levels/" + level + ".lvl");
    }
    catch (IOException e)
    {
      e.getMessage();
    }
  }
  
  public int getX()
  {
    return this.x;
  }
  
  public int getY()
  {
    return this.y;
  }
  
  public int getEndX()
  {
    return this.endX;
  }
  
  public int getEndY()
  {
    return this.endY;
  }
  
  public String getNextLevelName()
  {
    return this.next;
  }
  
  public String[] getEnemies()
  {
    return this.enemy;
  }
  
  private void parseFile(String filename)
    throws IOException
  {
    InputStream is = getClass().getClassLoader().getResourceAsStream(filename);
    BufferedReader reader = new BufferedReader(new InputStreamReader(is));
    for (;;)
    {
      String line = reader.readLine();
      if (line == null)
      {
        reader.close();
        return;
      }
      line = line.trim();
      if ((line.length() > 0) && (!line.startsWith("#"))) {
        try
        {
          parseLine(line);
        }
        catch (NumberFormatException e)
        {
          throw new IOException(e.getMessage());
        }
        catch (NoSuchElementException e)
        {
          throw new IOException(e.getMessage());
        }
      }
    }
  }
  
  private void parseLine(String line)
    throws IOException, NumberFormatException, NoSuchElementException
  {
    StringTokenizer tokenizer = new StringTokenizer(line);
    String command = tokenizer.nextToken();
    if (command.equals("N"))
    {
      this.next = tokenizer.nextToken();
    }
    else if (command.equals("X"))
    {
      this.x = Integer.parseInt(tokenizer.nextToken());
    }
    else if (command.equals("Y"))
    {
      this.y = Integer.parseInt(tokenizer.nextToken());
    }
    else if (command.equals("W"))
    {
      this.endX = Integer.parseInt(tokenizer.nextToken());
    }
    else if (command.equals("Z"))
    {
      this.endY = Integer.parseInt(tokenizer.nextToken());
      this.enemy = new String[this.endX * this.endY];
    }
    else if (command.equals("E"))
    {
      for (int i = this.tmpEnemyLineNo * this.endX; i < this.endX + this.endX * this.tmpEnemyLineNo; i++) {
        this.enemy[i] = tokenizer.nextToken();
      }
      this.tmpEnemyLineNo += 1;
    }
  }
}
