public class Timer
{
  private static Timer timer = new Timer();
  private long start = 0L;
  private long ticks = 0L;
  private boolean running = false;
  
  static
  {
    timer.start();
  }
  
  private static long timerTicksPerSecond = timer.getTicksPerSecond();
  private static long resolution;
  
  public static long getTime()
  {
    return timer.getClockTicks() * 1000L / timerTicksPerSecond;
  }
  
  public static void sleep(long duration)
  {
    timer.sleepUntil(duration * timerTicksPerSecond / 1000L);
  }
  
  public Timer()
  {
    if (resolution == 0L) {
      init();
    }
  }
  
  private void init()
  {
    long time = System.currentTimeMillis();
    long prev_time = time;
    while (time == prev_time) {
      time = System.currentTimeMillis();
    }
    resolution = time - prev_time;
    prev_time = time;
    while (time == prev_time) {
      time = System.currentTimeMillis();
    }
    if (time - prev_time < resolution) {
      resolution = time - prev_time;
    }
  }
  
  private long getTicksPerSecond()
  {
    if (resolution == 0L) {
      init();
    }
    return 1000L / resolution;
  }
  
  private long getClockTicks()
  {
    if (this.running) {
      this.ticks = ((System.currentTimeMillis() - this.start) / resolution);
    }
    return this.ticks;
  }
  
  private void start()
  {
    long time = System.currentTimeMillis();
    long prev_time = time;
    this.ticks = 0L;
    this.running = true;
    while (time == prev_time) {
      time = System.currentTimeMillis();
    }
    this.start = System.currentTimeMillis();
  }
  
  private void stop()
  {
    this.ticks = getClockTicks();
    this.running = false;
  }
  
  private void sleepUntil(long ticks)
    throws IllegalStateException
  {
    long tick = getClockTicks();
    if (!this.running) {
      throw new IllegalStateException("Timer not running!");
    }
    while (getClockTicks() < tick + ticks) {
      Thread.yield();
    }
  }
}
